# -*- coding: utf-8 -*-
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

class OS:
    LINUX = 'Linux'
    WINDOWS = 'Windows'
    FREE_BSD = 'FreeBSD'
    OPEN_BSD = 'OpenBSD'
    OSX = DARWIN = 'Darwin'
    BSD_FAMILY = [FREE_BSD, OPEN_BSD]
    UNIX_LIKE = [FREE_BSD, OPEN_BSD, LINUX]
    LIST = [LINUX, WINDOWS, FREE_BSD, OPEN_BSD, OSX]
